/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef _CPU_H_
#define _CPU_H_

#include "emuriver.h"

#define CPU_STOP_NORMAL 1
#define CPU_STOP_BREAK 2
#define CPU_STOP_RWATCH 3
#define CPU_STOP_WWATCH 4
#define CPU_STOP_AWATCH 5
#define CPU_STOP_TERMINATE 6
#define CPU_STOP_QUIT 7
#define CPU_STOP_ILLEGAL 8

struct breaks {
  int n, max_n;
  uint32 *tab;
};

struct cpu {
  uint32 reg[16];
  uint32 cpsr;
  uint32 spsr;
  uint32 saved_svc[3];
  uint32 saved_abt[3];
  uint32 saved_und[3];
  uint32 saved_irq[3];
  uint32 saved_fiq[8];
  struct memory *mem;
  struct breaks breakpoints;
  struct breaks rwatchpoints;
  struct breaks wwatchpoints;
  struct breaks awatchpoints;
  int stop;
  uint32 break_addr;
};

struct cpu *create_cpu(struct memory *memory);
int cpu_go(struct cpu *cpu, int n_instr);
void cpu_irq(struct cpu *cpu);

#define CPU_BRK_BREAK 1
#define CPU_BRK_RWATCH 2
#define CPU_BRK_WWATCH 3
#define CPU_BRK_AWATCH 4

void cpu_remove_break(struct cpu *cpu, int type, uint32 val);
int cpu_insert_break(struct cpu *cpu, int type, uint32 val);

#endif
