/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef _HARDWARE_H_
#define _HARDWARE_H_

#include "emuriver.h"
#include <stdio.h>

struct gpio_handler {
  void *data;
  int (*get)(void *data, int port, int bit);
  int (*set)(void *data, int port, int bit, int value);
};

struct hardware {
  uint32 gpio_dir[8];
  uint32 gpio_out[8];
  struct gpio_handler gpio_handler[8][32];

  struct flash *flash[4];
  struct prog_flash *prog_flash;

  struct lcd *lcd;
};

struct flash {
  uint8 reg[2112];
  int ptr;
  int adr_ptr;
  FILE *fp;

  int busy, ready_tick;

  uint8 adr[5];
  uint8 cmd;
};

struct prog_flash {
  int state;
  FILE *fp;
  uint8 *content;
};

extern unsigned int tick;

int tick_before(unsigned int a, unsigned int b);

int gpio_get_bit(struct hardware *hardware, int port, int bit);
void gpio_install_handler(struct hardware *hardware, int port, int bit,
			  void *data,
			  int (*get)(void *, int, int),
			  int (*set)(void *, int, int, int));

void install_hardware(struct hardware *hardware, struct memory *memory);
void hardware_reset(struct hardware *hardware);
struct hardware *create_hardware(void);

#endif
