/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <curses.h>

#include "lcd.h"

#define N_KEYS 10

static int key_timer[10];

struct key {
  int term_key;
  int iriver_key;
  int time;
};

static WINDOW *stdscr, *pad;
static pad_x, pad_y;

#define SHORT 10
#define LONG 80

static struct key keys[] = {
  {'q', 0, SHORT},
  {'1', 1, SHORT},
  {'!', 1, LONG},
  {'2', 2, SHORT},
  {'@', 2, LONG},
  {'3', 3, SHORT},
  {'#', 3, LONG},
  {' ', 4, SHORT},
  {'M', 4, LONG},
  {'j', 5, SHORT},
  {'J', 5, LONG},
  {'l', 6, SHORT},
  {'L', 6, LONG},
  {'i', 7, SHORT},
  {'I', 7, LONG},
  {'k', 8, SHORT},
  {'K', 8, LONG},
  {0, 0, 0}
};

int ui_init(void) {
  stdscr = initscr();
  cbreak();
  noecho();
  nonl();
  intrflush(stdscr, FALSE);
  keypad(stdscr, TRUE);
  nodelay(stdscr, 1);

  pad = newpad(LCD_HEIGHT, LCD_WIDTH);
  pad_x = pad_y = 0;

  return 0;
}

void ui_cleanup(void) {
  endwin();
}

static void keyboard_handle(void) {
  int ch, i;

  for (i = 0; i < N_KEYS; i++)
    if (key_timer[i]) key_timer[i]--;

  while ((ch = getch()) != ERR) {
    switch (ch) {
    case KEY_DOWN:
      pad_y += 16;
      break;

    case KEY_UP:
      pad_y -= 16;
      if (pad_y < 0) pad_y = 0;
      break;

    case KEY_LEFT:
      pad_x -= 16;
      if (pad_x < 0) pad_x = 0;
      break;

    case KEY_RIGHT:
      pad_x += 16;
      break;

    default:
      for (i = 0; keys[i].term_key; i++)
	if (ch == keys[i].term_key)
	  key_timer[keys[i].iriver_key] = keys[i].time;
      break;
    }
  }
}

void ui_refresh(struct lcd *lcd) {
  int i, w, h, x, y;

  for (y = 0; y < LCD_HEIGHT - 1; y++)
    for (x = 0; x < LCD_WIDTH; x++)
      mvwaddch(pad, y, x, lcd_get_pixel(lcd, x, y) ? '#' : '.');

  keyboard_handle();
  getmaxyx(stdscr, h, w);

  if (pad_x + w > LCD_WIDTH)
    pad_x = LCD_WIDTH - w;
  if (pad_y + h > LCD_HEIGHT)
    pad_y = LCD_HEIGHT - h;
  
  prefresh(pad, pad_y, pad_x, 0, 0, h - 2, w - 1);
}

int ui_get_key(int key) {
  return key_timer[key] > 0;
}
